<?php

namespace Drupal\plausible\Form;

use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Render\Markup;
use Drupal\Core\Url;

/**
 * Configure Plausible settings for this site.
 */
class PlausibleSettingsForm extends ConfigFormBase {

  /**
   * Config settings.
   *
   * @var string
   */
  const SETTINGS = 'plausible.settings';

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'plausible_admin_settings';
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames() {
    return [
      static::SETTINGS,
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $config = $this->config(static::SETTINGS);

    $intro_message = $this->t('Visit @plausibleWebsite to sign up for a free account.', [
      '@plausibleWebsite' => Markup::create('<a href="https://plausible.io" target="_blank">Plausible.io</a>'),
    ]);
    $form['intro'] = [
      '#markup' => '<p>' . $intro_message . '</p>',
    ];

    $front_url = Url::fromRoute('<front>')->setAbsolute()->toString();
    $current_domain = parse_url($front_url, PHP_URL_HOST);

    $form['script_domain'] = [
      '#default_value' => $config->get('script.domain'),
      '#type' => 'textfield',
      '#title' => $this->t('Domain'),
      '#description' => $this->t('The domain of your site as configured in Plausible. If this is left empty, the current domain will be used.'),
      '#placeholder' => $current_domain,
    ];

    $form['script_api'] = [
      '#default_value' => $config->get('script.api'),
      '#type' => 'textfield',
      '#title' => $this->t('API endpoint'),
      '#description' => $this->t('The endpoint where the data should be sent. If this is left empty, the default endpoint will be used.'),
    ];

    $form['script_src'] = [
      '#default_value' => $config->get('script.src'),
      '#type' => 'textfield',
      '#title' => $this->t('Script source'),
      '#description' => $this->t('The path or url of the tracking script'),
      '#required' => TRUE,
    ];

    // Visibility settings.
    $form['tracking_scope'] = [
      '#type' => 'vertical_tabs',
      '#title' => $this->t('Tracking scope'),
    ];

    // Pages.
    $form['tracking']['page_visibility_settings'] = [
      '#type' => 'details',
      '#title' => $this->t('Pages'),
      '#group' => 'tracking_scope',
    ];
    $description = $this->t("Specify pages by using their paths. Enter one path per line.
     The '*' character is a wildcard. Example paths are %blog for the blog page and
     %blog-wildcard for every personal blog. %front is the front page.", [
       '%blog' => '/blog',
       '%blog-wildcard' => '/blog/*',
       '%front' => '<front>',
     ]);

    $title = $this->t('Pages');
    $form['tracking']['page_visibility_settings']['visibility_request_path_mode'] = [
      '#type' => 'radios',
      '#title' => $this->t('Add tracking to specific pages'),
      '#options' => [
        $this->t('Disabled'),
        $this->t('Every page except the listed pages'),
        $this->t('The listed pages only'),
      ],
      '#default_value' => $config->get('visibility.request_path_mode'),
    ];
    $visibility_request_path_pages = $config->get('visibility.request_path_pages');
    $form['tracking']['page_visibility_settings']['visibility_request_path_pages'] = [
      '#type' => 'textarea',
      '#title' => $title,
      '#title_display' => 'invisible',
      '#default_value' => !empty($visibility_request_path_pages) ? $visibility_request_path_pages : '',
      '#description' => $description,
      '#rows' => 10,
      '#states' => [
        'visible' => [
          ':input[name="visibility_request_path_mode"]' => ['!value' => '0'],
        ],
      ],
    ];

    // Roles.
    $visibility_user_role_roles = $config->get('visibility.user_role_roles');
    $form['tracking']['role_visibility_settings'] = [
      '#type' => 'details',
      '#title' => $this->t('Roles'),
      '#group' => 'tracking_scope',
    ];

    $form['tracking']['role_visibility_settings']['visibility_user_role_mode'] = [
      '#type' => 'radios',
      '#title' => $this->t('Add tracking for specific roles'),
      '#options' => [
        $this->t('Disabled'),
        $this->t('Add to the selected roles only'),
        $this->t('Add to every role except the selected ones'),
      ],
      '#default_value' => $config->get('visibility.user_role_mode'),
    ];
    $form['tracking']['role_visibility_settings']['visibility_user_role_roles'] = [
      '#type' => 'checkboxes',
      '#title' => $this->t('Roles'),
      '#default_value' => !empty($visibility_user_role_roles) ? $visibility_user_role_roles : [],
      '#options' => array_map('\Drupal\Component\Utility\Html::escape', user_role_names()),
      '#description' => $this->t('If none of the roles are selected, all users will be tracked. If a user has any of the roles checked, that user will be tracked (or excluded, depending on the setting above).'),
      '#states' => [
        'visible' => [
          ':input[name="visibility_user_role_mode"]' => ['!value' => '0'],
        ],
      ],
    ];

    return parent::buildForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state) {
    $domains = explode(',', $form_state->getValue('script_domain'));
    $domain_validation = '/[a-zA-Z0-9\.\/\?\:@\-_=#]+\.[a-zA-Z0-9\&\.\/\?\:@\-_=#]{2,}/';

    foreach ($domains as $domain) {
      if ($domain === '') {
        continue;
      }

      if (!preg_match($domain_validation, $domain)) {
        $form_state->setErrorByName('script_domain', $this->t('%domain is not a valid domain. Please enter the full domain.', ['%domain' => $domain]));
      }
    }

    // Verify that every path is prefixed with a slash, but do not check for
    // slashes if no paths configured.
    if ($form_state->getValue('visibility_request_path_mode') != 2
      && !empty($form_state->getValue('visibility_request_path_pages'))) {

      $pages = preg_split('/(\r\n?|\n)/', $form_state->getValue('visibility_request_path_pages'));
      foreach ($pages as $page) {
        if (strpos($page, '/') !== 0 && $page !== '<front>') {
          $msg = $this->t('Path "@page" not prefixed with slash.', ['@page' => $page]);
          $form_state->setErrorByName('visibility_request_path_pages', $msg);
          // Drupal forms show one error only.
          break;
        }
      }
    }
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    // Retrieve the configuration.
    $this->configFactory->getEditable(static::SETTINGS)
      // Set the submitted configuration setting.
      ->set('script.domain', $form_state->getValue('script_domain'))
      ->set('script.api', $form_state->getValue('script_api'))
      ->set('script.src', $form_state->getValue('script_src'))
      ->set('visibility.request_path_mode', $form_state->getValue('visibility_request_path_mode'))
      ->set('visibility.request_path_pages', $form_state->getValue('visibility_request_path_pages'))
      ->set('visibility.user_role_mode', $form_state->getValue('visibility_user_role_mode'))
      ->set('visibility.user_role_roles', $form_state->getValue('visibility_user_role_roles'))
      ->save();

    parent::submitForm($form, $form_state);
  }

}
